#!/usr/bin/env python

# Copyright and license: Please refer to LICENSE.TXT

import sys
import os
import os.path
import string
import re
import time
import sqlite3
from datetime import timedelta
from datetime import datetime
from ftplib import FTP

_selfpath=(os.path.dirname(os.path.realpath(__file__)).split('/'))[:-1]
_selfpath.append('pylibs')
_libspath='/'.join(_selfpath)
sys.path.append(_libspath)

from castiron import *

sys.path.append(CORE_PATH_BASE+'/pylibs')

from s2p import *
from s2p_logger import *

#############################################################################################################
# Check config parameters

IGX_PATH_CONFIG = APPS_PATH_BASE+'/igx/config'
IGX_CONF_SYSTEM = xml_to_array(IGX_PATH_CONFIG+'/system.xml')

CORE_CONF_SYSTEM		= xml_to_array(CORE_PATH_BASE+'/config/system.xml')
CASTIRON_CONF_BACKUP	= xml_to_array(CASTIRON_PATH_BASE+'/config/backup.xml')

if CASTIRON_CONF_BACKUP['callslog']['enabled'] != 'true': sys.exit(0)

#############################################################################################################
# Check backup frequency

frequency=CASTIRON_CONF_BACKUP['callslog']['frequency']

do_backup=False

if len(sys.argv)<2 or sys.argv[1] != 'cron':
	s2p_log("castiron", "info", "Manually creating calls log backup")
	do_backup=True

if len(sys.argv)>1 and sys.argv[1] == 'cron' and frequency == 'manual':
	sys.exit(0)

ltime=time.localtime()

hours_m		= re.match(r'^(?P<n>\d+)h$', frequency)
days_m		= re.match(r'^(?P<n>\d+)d$', frequency)
weeks_m		= re.match(r'^(?P<n>\d+)w$', frequency)
months_m	= re.match(r'^(?P<n>\d+)m$', frequency)

if hours_m:
	ds=time.time()-3600*int(hours_m.group('n'))
	date_start=time.strftime("%Y%m%d%H0000",  time.localtime(ds))
	if not (ltime[3] % int(hours_m.group('n'))): do_backup=True

elif days_m:
	ds=time.time()-24*3600*int(days_m.group('n'))
	date_start=time.strftime("%Y%m%d000000",  time.localtime(ds))
	if not (ltime[7] % int(days_m.group('n'))) and not int(ltime[3]): do_backup=True

elif weeks_m:
	ds=time.localtime()-7*24*3600*int(weeks_m.group('n'))
	date_start=time.strftime("%Y%m%d000000",  time.localtime(ds))
	if not (ltime[6] % int(weeks_m.group('n'))) and not int(ltime[3]): do_backup=True

elif months_m:
	ds=time.localtime()-20*24*3600*int(months_m.group('n'))
	date_start=time.strftime("%Y%m01000000",  time.localtime(ds))
	if not (ltime[1] % int(months_m.group('n'))) and not int(ltime[3]) and int(ltime[2]) == 1: do_backup=True

else:
	ds=0

if not do_backup: sys.exit(0)

#############################################################################################################
# Do backup

timestamp=time.strftime("%Y%m%d%H%M%S", time.localtime())
log_file_uncompressed=S2P_PATH_BASE+'/tmp/calls_log-'+timestamp+'.csv'

fp=open(log_file_uncompressed, 'w')

IGX_DATABASE=sqlite3.connect(IGX_PATH_CONFIG+'/'+IGX_CONF_SYSTEM['database'], check_same_thread=False)

cur = IGX_DATABASE.cursor()
tablename=IGX_CONF_SYSTEM['callog']['db-table']

headers=['date', 'time', 'chan_from', 'chan_to', 'skype_remote', 'skype_local', 'status', 'user_id' ,'duration', 'direction', 'rate', 'rate_cur']
cur.execute("select "+(','.join(headers))+" from "+tablename+" where date>'"+date_start+"'")

fp.write((','.join(headers))+"\n")
for row in cur:
	values=[]

	for i in row: values.append('"'+str(i)+'"')
	fp.write((','.join(values))+"\n")

fp.close()

#############################################################################################################
# Compression

if CASTIRON_CONF_BACKUP['callslog']['compress'] == 'true':
	os.system('gzip '+log_file_uncompressed)
	log_file=log_file_uncompressed+'.gz'
	
else:
	log_file=log_file_uncompressed


#############################################################################################################
# Email variables

# Variables
fp=open('/etc/hostname', 'r')
hostname=re.sub(r'[\n\r]+', '', fp.read())
fp.close()

sys_vars={
	'sysname':			CORE_CONF_SYSTEM['hq']['sysname'],
	'server':			hostname,
	'hostname':			hostname,
	'date-time':		time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
}

#############################################################################################################
# Send backup

subject	= set_vars(CASTIRON_CONF_BACKUP['callslog']['message-template-subject'], sys_vars)
body	= set_vars(CASTIRON_CONF_BACKUP['callslog']['message-template-body'], sys_vars)

res=ci_media_send({
	'media': 	CASTIRON_CONF_BACKUP['callslog']['media'],
	'subject':	subject,
	'body':		body,
	'files':	[log_file]
})

os.unlink(log_file)

if res:
	s2p_log("castiron", "info", "Calls log backup sent")
	sys.exit(0)
	
else:
	s2p_log("castiron", "error", "Could not send calls log backup")
	sys.exit(-1)




