<?php
	require_once(CI_PATH_LIBS.'/config.inc.php');
	
	class CI_USERS extends CI_CONFIG
	{
		function CI_USERS()
		{
		}
		
		function get_config_file()
		{
			return IGX_PATH_BASE.'/config/auth/users.xml';
		}

		function get_rmap($info)
		{
			$info['id']=preg_replace("/[^\w\-_]+/", '', $info['id']);
			$info['id']=trim($info['id']);
			
			return array(
				'id'	=> $info['id'],
				'desc'	=> $info['desc'],
				'userdata'	=> array(
					'castiron-webaccess'	=> $info['ud_castiron_webaccess'],
					'castiron-role'			=> $info['ud_castiron_role'],
					'castiron-password'		=> $info['ud_castiron_password'],
					'castiron-email'		=> $info['ud_castiron_email']
				),
				'enabled'	=> $info['enabled'],
				'group'		=> $info['group'],
				'user'		=> $info['user'],
				'password'	=> $info['password']
			);
		}

		function get_map($info)
		{
			$info['id']=preg_replace("/[^\w\-_]+/", '', $info['id']);
			$info['id']=trim($info['id']);
			
			return array(
				'id'					=> $info['id'],
				'desc'					=> $info['desc'],
				'ud_castiron_webaccess'	=> $info['userdata']['castiron-webaccess'],
				'ud_castiron_role'		=> $info['userdata']['castiron-role'],
				'ud_castiron_password'	=> $info['userdata']['castiron-password'],
				'ud_castiron_email'		=> $info['userdata']['castiron-email'],
				'enabled'				=> $info['enabled'],
				'group'					=> $info['group'],
				'user'					=> $info['user'],
				'password'				=> $info['password']
			);
		}

		function get_info($id)
		{
			$info =& parent::get_info($id);
			if (sizeof($info)) return $info;

			/* Empty (default values) */
			return array(
				'enabled'				=> 'true',
				'ud_castiron_webaccess'	=> 'true',
				'ud_castiron_role'		=> 'user'
			);
		}

		function search_pin($pin)
		{
			$list =& $this->get_raw_list();
			for ($i=0; $i<sizeof($list); $i++)
			{
				if (!$list[$i]['user']) continue;
				if ($list[$i]['user'] == $pin) return $list[$i]['id'];
			}

			return false;
		}

		function task_toggle($id)
		{
			/* Filter params */
			$id=preg_replace("/[^\w\-_]+/", '', $id);
			$id=trim($id);

			if ($id == 'admin') return ci_response_error(
				'field',
				array(
					'fieldname' => 'id',
					'desc'		=> ci_lbl('libusers.error.invalid-for-admin')
				)
			);

			return parent::task_toggle($id);
		}

		function task_delete($id)
		{
			global $CI_DB;
			
			/* Filter params */
			$id=preg_replace("/[^\w\-_]+/", '', $id);
			$id=trim($id);

			if ($id == 'admin') return ci_response_error(
				'field',
				array(
					'fieldname' => 'id',
					'desc'		=> ci_lbl('libusers.error.invalid-for-admin')
				)
			);

			/* Remove group's addressbook */
			$addressbook=new CI_ADDRESSBOOK();
			@unlink($addressbook->get_config_file($addressbook->get_folder_name('user', $id)));
			$CI_DB->query("delete from addressbook where user_id=".$CI_DB->quote($id));

			return parent::task_delete($id);
		}

		function task_save($info)
		{
			global $CI_DB;
			
			/* Filter params */
			$info['id']=preg_replace("/[^\w\-_]+/", '', $info['id']);
			$info['id']=trim($info['id']);

			if (!$info['id']) return ci_response_error(
					'field',
					array(
						'fieldname' => 'id',
						'desc'		=> ci_lbl('libusers.error.invalid-username')
					)
				);

			/* Read old info */
			$old_info=$this->get_info($info['id']);

			/* Validate */
			if ($old_info['id'] && ($info['id'] != $old_info['id']))
				return ci_response_error(
					'id',
					array(
						'fieldname' => 'id',
						'desc'		=> ci_lbl('libusers.error.duplicated-username')
					)
				);

			$foo=$this->search_pin($info['user']);
			if ($foo && ($info['id'] != $foo))
				return ci_response_error(
					'field',
					array(
						'fieldname' => 'user',
						'desc'		=> ci_lbl('libusers.error.duplicated-pinuser')
					)
				);

			/* Compose fields */
			$fields=$this->get_rmap($info);
			$fields['userdata']['castiron-password']=$info['ud_castiron_password'] ? md5($info['ud_castiron_password']) : $old_info['ud_castiron_password'];
			
			/* Admin special features */
			if ($info['id'] == 'admin')
			{
				$fields['group']='admin';
				$fields['enabled']='true';
				$fields['userdata']['castiron-webaccess']='true';
				$fields['userdata']['castiron-role']='admin';
			}

			/* Retrieve item position */
			$n=$this->get_n($info['id']);

			/* Save */
			$list =& $this->get_raw_list();
			$list[$n]=$fields;
			
			$this->save_raw_config($list);

			/* Create user's addressbook */
			$addressbook=new CI_ADDRESSBOOK();
			$addressbook_file=$addressbook->get_folder_name('user', $info['id']);
			if (!file_exists($addressbook_file))
			{
				$addressbook=new CI_ADDRESSBOOK();
				$foo=array(
					'default'	=> '${to}',
					'sql'	=> array(
						'db-query'	=> "select destination from addressbook where shortcut='\${to}' and user_id=".$CI_DB->quote($info['id'])
					),
					'items'		=> ''
				);
				
				ci_save_xml($addressbook->get_config_file($addressbook_file), $foo, 'map');
			}

			
			return ci_response_ok();
		}
	}
?>