#!/usr/bin/env python

# Copyright and license: Please refer to LICENSE.TXT

import sys
import os
import os.path
import string
import re
import time
from xml.sax import saxutils

_selfpath=(os.path.dirname(os.path.realpath(__file__)).split('/'))[:-1]
_selfpath.append('pylibs')
_libspath='/'.join(_selfpath)
sys.path.append(_libspath)
sys.path.append(_libspath+'/pythondialog')

import dialog
from castiron import *

# Configuration files
CORE_CONF_SYSTEM 		= xml_to_array(CORE_PATH_BASE+'/config/system.xml')
CASTIRON_CONF_SYSTEM	= xml_to_array(CASTIRON_PATH_BASE+'/config/system.xml')

# Import language
execfile(CASTIRON_PATH_BASE+'/langs/'+CORE_CONF_SYSTEM['language']+'/labels.py')

# Scrittura file di configurazione
def set_netconfig(iface, dhcp, ip='', nm='', gw=''):
	fp=open("/etc/network/interfaces", "r")

	current_iface=''
	out_file=''

	configured={}
	
	for l in fp:
		line=re.sub(r'[\n\r]+', '', l)

		if not line or line[0] == '#': continue

		m=re.match(r'^auto\s+(?P<ifname>\w+)\s*$', line)
		m2=re.search(r'^iface\s+(?P<ifname>\w+)\s', line)
		
		if m or m2:
			if m: current_iface		= m.group('ifname')
			if m2: current_iface	= m2.group('ifname')

		if current_iface==iface:
			
			if not configured.has_key(current_iface):
				configured[current_iface]=True

				out_file=out_file+"auto "+iface+"\n"

				if dhcp:
					out_file=out_file+"iface "+iface+" inet dhcp\n"
					
				else:
					out_file=out_file+"iface "+iface+" inet static\n"
					out_file=out_file+"\taddress "+ip+"\n"
					out_file=out_file+"\tnetmask "+nm+"\n"

					if len(gw): out_file=out_file+"\tgateway "+gw+"\n"
			
		else:
			out_file=out_file+line+"\n"

	fp.close()

	fp=open("/etc/network/interfaces", "w")
	fp.write(out_file)
	fp.close()

	stop_network()
	start_network()

def get_iface_ip(eth):
	return systempt("ifconfig "+eth+"|grep \"inet addr\"|awk '{print $2}'|cut -d: -f2")

def get_iface_nm(eth):
	return systempt("ifconfig "+eth+"|grep \"inet addr\"|awk '{print $4}'|cut -d: -f2")

def list_ifaces():
	return re.split(r'[\n\r]+',  systempt("ifconfig -a|grep ^eth|awk '{print $1}'"))

def get_gw():
	return systempt("route -n|awk '$4 == \"UG\" {print $2;}'")

def ci_lbl(code):
	if not CI_LBL.has_key(code): return code
	return CI_LBL[code]

def is_valid_ip(ip):
	if re.match(r"^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$", ip):
		return 1
	
	return 0

def stop_network():
	d.infobox(ci_lbl('terminal.network.stop'), width=70)
	os.system("/etc/init.d/networking stop 1>/dev/null 2>/dev/null")

def start_network():
	d.infobox(ci_lbl('terminal.network.start'), width=70)
	os.system("/etc/init.d/networking start 1>/dev/null 2>/dev/null")
	rebuild_hosts()

def menu_ip(iface):
	new_ip=get_iface_ip(iface)
	new_nm=get_iface_nm(iface)
	new_gw=get_gw()

	valid=0
	while not valid:

		use_dhcp=0
		code = d.yesno(ci_lbl('terminal.network.usedhcp'), width=40)

		if code == 0:
			use_dhcp=1
			valid=1

		else:
		
			(code, tag) = d.inputbox(ci_lbl('terminal.network.askip'), width=40, init=new_ip, cancel=ci_lbl('terminal.cancel'))
			if int(code) != 0: return

			new_ip=tag
			
			if new_nm=="": new_nm="255.255.255.0"
			(code, tag) = d.inputbox(ci_lbl('terminal.network.asknm'), width=40, init=new_nm, cancel=ci_lbl('terminal.back'))
			if int(code) != 0: continue

			new_nm=tag

			if new_gw=="": new_gw="192.168.0.254"
			(code, tag) = d.inputbox(ci_lbl('terminal.network.askgw'), width=40, init=new_gw, cancel=ci_lbl('terminal.back'))
			if int(code) != 0: continue

			new_gw=tag

			if is_valid_ip(new_ip) and is_valid_ip(new_nm) and is_valid_ip(new_gw):
				valid=1
			else:
				d.msgbox(ci_lbl('terminal.network.checkdata'))

		if use_dhcp:
			code=d.yesno(ci_lbl('terminal.network.confirm')+"\n\nIP: dhcp assigned\nNetmask: dhcp assigned\nGateway: dhcp assigned", width=50)
			if code != 0: valid=0
			
		else:
			# Richiesta conferma
			code=d.yesno(ci_lbl('terminal.network.confirm')+"\n\nIP: "+new_ip+"\nNetmask: "+new_nm+"\nGateway: "+new_gw, width=50)
			if code != 0: valid=0
	
	if use_dhcp:	set_netconfig(iface, use_dhcp)
	else:			set_netconfig(iface, 0, new_ip, new_nm, new_gw)

def rebuild_hosts():
	fp=open("/etc/hostname", "r")
	hostname=re.sub(r"[\r\n]+", '', fp.readline())
	fp.close()

	domain=""
	fp=open("/etc/resolv.conf", "r")
	for line in fp:
		m=re.match("^search\s+(.+)$", line)
		if m: domain=m.group(1)
	fp.close()
	
	fp=open("/etc/hosts", "w")
	fp.write("127.0.0.1\tlocalhost\tlocalhost.localdomain\t"+hostname+"\n")
	fp.close()

def menu_hngwdns():

	while 1:
		choices=[]
		data={}
	
		data['hostname']=systempt("/bin/hostname")
		data['domain']=systempt("cat /etc/resolv.conf|awk '$1==\"search\" { print $2; exit }'")
		ns=systempt("cat /etc/resolv.conf|awk '$1==\"nameserver\" { print $2 }'").split("\n")
	
		if len(ns) < 1: ns.append("")
		if len(ns) < 2: ns.append("")
		if len(ns) < 3: ns.append("")
		data['dns1']=ns[0]
		data['dns2']=ns[1]
		data['dns3']=ns[2]
		
		choices.append(("hostname", data['hostname']))
		choices.append(("domain", 	data['domain']))
		choices.append(("dns1", 	data['dns1']))
		choices.append(("dns2", 	data['dns2']))
		choices.append(("dns3", 	data['dns3']))

		(code, tag) = d.menu(
			ci_lbl('terminal.select'),
			width=70,
			height=15,
			choices=choices
				)

		if int(code) != 0: return

		(code, tag2)=d.inputbox(ci_lbl('terminal.inputnew')+" "+tag, width=40, init=data[tag])
		if int(code) != 0: continue
		data[tag]=tag2

		if tag == "hostname":
			os.system("/bin/hostname "+data['hostname'])
			fp=open("/etc/hostname", "w")
			fp.write(data['hostname'])
			fp.close()

			rebuild_hosts()

		else:
			fp=open("/etc/resolv.conf", "w")
			fp.write("search "+data['domain']+"\n")
			if (data['dns1']): fp.write("nameserver "+data['dns1']+"\n")
			if (data['dns2']): fp.write("nameserver "+data['dns2']+"\n")
			if (data['dns3']): fp.write("nameserver "+data['dns3']+"\n")
			fp.close()

# Start dialog
d=dialog.Dialog(dialog="dialog")
d.add_persistent_args(["--backtitle", ci_lbl('terminal.title')])
d.add_persistent_args(["--cancel-label", ci_lbl('terminal.close')])

while 1:
	ifaces=list_ifaces()

	list_ch=[]
	for iface in ifaces:
		list_ch.append((iface, ci_lbl('terminal.network.ip')+' - '+iface))

	list_ch.append(("H", ci_lbl('terminal.network.hd')))
	list_ch.append(("F", ci_lbl('terminal.network.finished')))
	
	(code, tag) = d.menu(
		ci_lbl('terminal.select'),
		width=70,
		height=15,
		choices=list_ch
			)

	if int(code) != 0: break

	if (tag == "H"):	menu_hngwdns()
	elif (tag == "F"):	break
	else:				menu_ip(tag)

